/**
 * Declares the cardinality restriction data structure and related API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_CARDINALITY_ROLE_H
#define TME_CARDINALITY_ROLE_H

#include "tme_atomic.h"

TME_BEGIN_DECLS

/**
 * Infinite cardinality.
 *
 * @public @related TmeCardinalityRole
 */
#define TME_CARDINALITY_INF TME_UINT_MAX

/**
 * Models cardinality restrictions.
 *
 * @see TmeSemDesc
 */
typedef tme_struct(TmeCardinalityRole) {

    /// Role this restriction applies to.
    TmeAtomicRole atomic_role;

    /// Cardinality of this restriction.
    tme_uint_t cardinality;

} TmeCardinalityRole;

/// Vector of cardinality restrictions.
VECTOR_DECL_EQUATABLE_SPEC(TmeCardinalityRole, TME_PUBLIC)

/**
 * Equality function for cardinality restrictions.
 *
 * @param role LHS of the equality relation.
 * @param other_role RHS of the equality relation.
 * @return True if the equality relation holds, false otherwise.
 *
 * @public @memberof TmeCardinalityRole
 */
TME_PUBLIC
bool tme_cardinality_role_equals(TmeCardinalityRole role, TmeCardinalityRole other_role);

TME_END_DECLS

#endif // TME_CARDINALITY_ROLE_H
