/**
 * Defines metadata for the reasoner.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_REASONER_METADATA_H
#define TME_REASONER_METADATA_H

#include "tme_compat.h"

TME_BEGIN_DECLS

/// Reasoner name type.
typedef tme_struct(TmeReasonerName) {

    /// Brief name.
    char const *brief;

    /// Extended name.
    char const *extended;

} TmeReasonerName;

/// Reasoner vendor type.
typedef tme_struct(TmeReasonerVendor) {

    /// Vendor name.
    char const *name;

    /// Vendor organization.
    char const *organization;

    /// Vendor URL.
    char const *url;

} TmeReasonerVendor;

/// Reasoner version type.
typedef tme_struct(TmeReasonerVersion) {

    /// Major revision.
    int major;

    /// Minor revision.
    int minor;

    /// Patch number.
    int patch;

} TmeReasonerVersion;

/**
 * Reasoner name.
 *
 * @public @related TmeReasonerName
 */
extern TmeReasonerName const tme_reasoner_name;

/**
 * Reasoner vendor.
 *
 * @public @related TmeReasonerVendor
 */
extern TmeReasonerVendor const tme_reasoner_vendor;

/**
 * Reasoner version.
 *
 * @public @related TmeReasonerVersion
 */
extern TmeReasonerVersion const tme_reasoner_version;

TME_END_DECLS

#endif // TME_REASONER_METADATA_H
